# AUTO-GENERATED FILE - DO NOT EDIT
# This file was automatically generated by the XDK build tool.
# Any manual changes will be overwritten on the next generation.

"""
Auto-generated direct messages client for the X API.

This module provides a client for interacting with the direct messages endpoints of the X API.

All methods, parameters, and response models are generated from the OpenAPI specification.

Generated automatically - do not edit manually.
"""

from __future__ import annotations
from typing import Dict, List, Optional, Any, Union, cast, TYPE_CHECKING
import requests
import time


if TYPE_CHECKING:
    from ..client import Client
from .models import (
    CreateByParticipantIdRequest,
    CreateByParticipantIdResponse,
    CreateByConversationIdRequest,
    CreateByConversationIdResponse,
    GetEventsResponse,
    CreateConversationRequest,
    CreateConversationResponse,
    GetEventsByConversationIdResponse,
    GetEventsByIdResponse,
    DeleteEventsResponse,
    GetEventsByParticipantIdResponse,
)


class DirectMessagesClient:
    """Client for direct messages operations"""


    def __init__(self, client: Client):
        self.client = client


    def create_by_participant_id(
        self, participant_id: Any, body: Optional[CreateByParticipantIdRequest] = None
    ) -> Dict[str, Any]:
        """
        Create DM message by participant ID
        Sends a new direct message to a specific participant by their ID.
        Args:
            participant_id: The ID of the recipient user that will receive the DM.
            body: Request body
        Returns:
            CreateByParticipantIdResponse: Response data
        """
        url = (
            self.client.base_url + "/2/dm_conversations/with/{participant_id}/messages"
        )
        url = url.replace("{participant_id}", str(participant_id))
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        headers = {}
        headers["Content-Type"] = "application/json"
        # Prepare request data
        json_data = None
        if body is not None:
            json_data = (
                body.model_dump(exclude_none=True)
                if hasattr(body, "model_dump")
                else body
            )
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        else:
            response = self.client.session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return CreateByParticipantIdResponse.model_validate(response_data)


    def create_by_conversation_id(
        self,
        dm_conversation_id: str,
        body: Optional[CreateByConversationIdRequest] = None,
    ) -> Dict[str, Any]:
        """
        Create DM message by conversation ID
        Sends a new direct message to a specific conversation by its ID.
        Args:
            dm_conversation_id: The DM Conversation ID.
            body: Request body
        Returns:
            CreateByConversationIdResponse: Response data
        """
        url = self.client.base_url + "/2/dm_conversations/{dm_conversation_id}/messages"
        url = url.replace("{dm_conversation_id}", str(dm_conversation_id))
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        headers = {}
        headers["Content-Type"] = "application/json"
        # Prepare request data
        json_data = None
        if body is not None:
            json_data = (
                body.model_dump(exclude_none=True)
                if hasattr(body, "model_dump")
                else body
            )
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        else:
            response = self.client.session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return CreateByConversationIdResponse.model_validate(response_data)


    def get_events(
        self,
        max_results: int = None,
        pagination_token: Any = None,
        event_types: List = None,
        dm_event_fields: List = None,
        expansions: List = None,
        media_fields: List = None,
        user_fields: List = None,
        tweet_fields: List = None,
    ) -> GetEventsResponse:
        """
        Get DM events
        Retrieves a list of recent direct message events across all conversations.
        Args:
            max_results: The maximum number of results.
            pagination_token: This parameter is used to get a specified 'page' of results.
            event_types: The set of event_types to include in the results.
            dm_event_fields: A comma separated list of DmEvent fields to display.
            expansions: A comma separated list of fields to expand.
            media_fields: A comma separated list of Media fields to display.
            user_fields: A comma separated list of User fields to display.
            tweet_fields: A comma separated list of Tweet fields to display.
            Returns:
            GetEventsResponse: Response data
        """
        url = self.client.base_url + "/2/dm_events"
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        if max_results is not None:
            params["max_results"] = max_results
        if pagination_token is not None:
            params["pagination_token"] = pagination_token
        if event_types is not None:
            params["event_types"] = ",".join(str(item) for item in event_types)
        if dm_event_fields is not None:
            params["dm_event.fields"] = ",".join(str(item) for item in dm_event_fields)
        if expansions is not None:
            params["expansions"] = ",".join(str(item) for item in expansions)
        if media_fields is not None:
            params["media.fields"] = ",".join(str(item) for item in media_fields)
        if user_fields is not None:
            params["user.fields"] = ",".join(str(item) for item in user_fields)
        if tweet_fields is not None:
            params["tweet.fields"] = ",".join(str(item) for item in tweet_fields)
        headers = {}
        # Prepare request data
        json_data = None
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.get(
                url,
                params=params,
                headers=headers,
            )
        else:
            response = self.client.session.get(
                url,
                params=params,
                headers=headers,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return GetEventsResponse.model_validate(response_data)


    def create_conversation(
        self, body: Optional[CreateConversationRequest] = None
    ) -> Dict[str, Any]:
        """
        Create DM conversation
        Initiates a new direct message conversation with specified participants.
        body: Request body
        Returns:
            CreateConversationResponse: Response data
        """
        url = self.client.base_url + "/2/dm_conversations"
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        headers = {}
        headers["Content-Type"] = "application/json"
        # Prepare request data
        json_data = None
        if body is not None:
            json_data = (
                body.model_dump(exclude_none=True)
                if hasattr(body, "model_dump")
                else body
            )
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        else:
            response = self.client.session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return CreateConversationResponse.model_validate(response_data)


    def get_events_by_conversation_id(
        self,
        id: Any,
        max_results: int = None,
        pagination_token: Any = None,
        event_types: List = None,
        dm_event_fields: List = None,
        expansions: List = None,
        media_fields: List = None,
        user_fields: List = None,
        tweet_fields: List = None,
    ) -> GetEventsByConversationIdResponse:
        """
        Get DM events for a DM conversation
        Retrieves direct message events for a specific conversation.
        Args:
            id: The DM conversation ID.
            max_results: The maximum number of results.
            pagination_token: This parameter is used to get a specified 'page' of results.
            event_types: The set of event_types to include in the results.
            dm_event_fields: A comma separated list of DmEvent fields to display.
            expansions: A comma separated list of fields to expand.
            media_fields: A comma separated list of Media fields to display.
            user_fields: A comma separated list of User fields to display.
            tweet_fields: A comma separated list of Tweet fields to display.
            Returns:
            GetEventsByConversationIdResponse: Response data
        """
        url = self.client.base_url + "/2/dm_conversations/{id}/dm_events"
        url = url.replace("{id}", str(id))
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        if max_results is not None:
            params["max_results"] = max_results
        if pagination_token is not None:
            params["pagination_token"] = pagination_token
        if event_types is not None:
            params["event_types"] = ",".join(str(item) for item in event_types)
        if dm_event_fields is not None:
            params["dm_event.fields"] = ",".join(str(item) for item in dm_event_fields)
        if expansions is not None:
            params["expansions"] = ",".join(str(item) for item in expansions)
        if media_fields is not None:
            params["media.fields"] = ",".join(str(item) for item in media_fields)
        if user_fields is not None:
            params["user.fields"] = ",".join(str(item) for item in user_fields)
        if tweet_fields is not None:
            params["tweet.fields"] = ",".join(str(item) for item in tweet_fields)
        headers = {}
        # Prepare request data
        json_data = None
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.get(
                url,
                params=params,
                headers=headers,
            )
        else:
            response = self.client.session.get(
                url,
                params=params,
                headers=headers,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return GetEventsByConversationIdResponse.model_validate(response_data)


    def get_events_by_id(
        self,
        event_id: Any,
        dm_event_fields: List = None,
        expansions: List = None,
        media_fields: List = None,
        user_fields: List = None,
        tweet_fields: List = None,
    ) -> GetEventsByIdResponse:
        """
        Get DM event by ID
        Retrieves details of a specific direct message event by its ID.
        Args:
            event_id: dm event id.
            dm_event_fields: A comma separated list of DmEvent fields to display.
            expansions: A comma separated list of fields to expand.
            media_fields: A comma separated list of Media fields to display.
            user_fields: A comma separated list of User fields to display.
            tweet_fields: A comma separated list of Tweet fields to display.
            Returns:
            GetEventsByIdResponse: Response data
        """
        url = self.client.base_url + "/2/dm_events/{event_id}"
        url = url.replace("{event_id}", str(event_id))
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        if dm_event_fields is not None:
            params["dm_event.fields"] = ",".join(str(item) for item in dm_event_fields)
        if expansions is not None:
            params["expansions"] = ",".join(str(item) for item in expansions)
        if media_fields is not None:
            params["media.fields"] = ",".join(str(item) for item in media_fields)
        if user_fields is not None:
            params["user.fields"] = ",".join(str(item) for item in user_fields)
        if tweet_fields is not None:
            params["tweet.fields"] = ",".join(str(item) for item in tweet_fields)
        headers = {}
        # Prepare request data
        json_data = None
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.get(
                url,
                params=params,
                headers=headers,
            )
        else:
            response = self.client.session.get(
                url,
                params=params,
                headers=headers,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return GetEventsByIdResponse.model_validate(response_data)


    def delete_events(self, event_id: Any) -> DeleteEventsResponse:
        """
        Delete DM event
        Deletes a specific direct message event by its ID, if owned by the authenticated user.
        Args:
            event_id: The ID of the direct-message event to delete.
            Returns:
            DeleteEventsResponse: Response data
        """
        url = self.client.base_url + "/2/dm_events/{event_id}"
        url = url.replace("{event_id}", str(event_id))
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        headers = {}
        # Prepare request data
        json_data = None
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.delete(
                url,
                params=params,
                headers=headers,
            )
        else:
            response = self.client.session.delete(
                url,
                params=params,
                headers=headers,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return DeleteEventsResponse.model_validate(response_data)


    def get_events_by_participant_id(
        self,
        participant_id: Any,
        max_results: int = None,
        pagination_token: Any = None,
        event_types: List = None,
        dm_event_fields: List = None,
        expansions: List = None,
        media_fields: List = None,
        user_fields: List = None,
        tweet_fields: List = None,
    ) -> GetEventsByParticipantIdResponse:
        """
        Get DM events for a DM conversation
        Retrieves direct message events for a specific conversation.
        Args:
            participant_id: The ID of the participant user for the One to One DM conversation.
            max_results: The maximum number of results.
            pagination_token: This parameter is used to get a specified 'page' of results.
            event_types: The set of event_types to include in the results.
            dm_event_fields: A comma separated list of DmEvent fields to display.
            expansions: A comma separated list of fields to expand.
            media_fields: A comma separated list of Media fields to display.
            user_fields: A comma separated list of User fields to display.
            tweet_fields: A comma separated list of Tweet fields to display.
            Returns:
            GetEventsByParticipantIdResponse: Response data
        """
        url = (
            self.client.base_url + "/2/dm_conversations/with/{participant_id}/dm_events"
        )
        url = url.replace("{participant_id}", str(participant_id))
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        if max_results is not None:
            params["max_results"] = max_results
        if pagination_token is not None:
            params["pagination_token"] = pagination_token
        if event_types is not None:
            params["event_types"] = ",".join(str(item) for item in event_types)
        if dm_event_fields is not None:
            params["dm_event.fields"] = ",".join(str(item) for item in dm_event_fields)
        if expansions is not None:
            params["expansions"] = ",".join(str(item) for item in expansions)
        if media_fields is not None:
            params["media.fields"] = ",".join(str(item) for item in media_fields)
        if user_fields is not None:
            params["user.fields"] = ",".join(str(item) for item in user_fields)
        if tweet_fields is not None:
            params["tweet.fields"] = ",".join(str(item) for item in tweet_fields)
        headers = {}
        # Prepare request data
        json_data = None
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.get(
                url,
                params=params,
                headers=headers,
            )
        else:
            response = self.client.session.get(
                url,
                params=params,
                headers=headers,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return GetEventsByParticipantIdResponse.model_validate(response_data)



# AUTO-GENERATED FILE - DO NOT EDIT
# This file was automatically generated by the XDK build tool.
# Any manual changes will be overwritten on the next generation.

"""
Auto-generated media client for the X API.

This module provides a client for interacting with the media endpoints of the X API.

All methods, parameters, and response models are generated from the OpenAPI specification.

Generated automatically - do not edit manually.
"""

from __future__ import annotations
from typing import Dict, List, Optional, Any, Union, cast, TYPE_CHECKING
import requests
import time


if TYPE_CHECKING:
    from ..client import Client
from .models import (
    GetUploadStatusResponse,
    UploadRequest,
    UploadResponse,
    GetAnalyticsResponse,
    CreateSubtitlesRequest,
    CreateSubtitlesResponse,
    DeleteSubtitlesRequest,
    DeleteSubtitlesResponse,
    CreateMetadataRequest,
    CreateMetadataResponse,
    GetByKeysResponse,
    InitializeUploadRequest,
    InitializeUploadResponse,
    FinalizeUploadResponse,
    GetByKeyResponse,
    AppendUploadRequest,
    AppendUploadResponse,
)


class MediaClient:
    """Client for media operations"""


    def __init__(self, client: Client):
        self.client = client


    def get_upload_status(
        self, media_id: Any, command: str = None
    ) -> GetUploadStatusResponse:
        """
        Get Media upload status
        Retrieves the status of a Media upload by its ID.
        Args:
            media_id: Media id for the requested media upload status.
            command: The command for the media upload request.
            Returns:
            GetUploadStatusResponse: Response data
        """
        url = self.client.base_url + "/2/media/upload"
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        if media_id is not None:
            params["media_id"] = media_id
        if command is not None:
            params["command"] = command
        headers = {}
        # Prepare request data
        json_data = None
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.get(
                url,
                params=params,
                headers=headers,
            )
        else:
            response = self.client.session.get(
                url,
                params=params,
                headers=headers,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return GetUploadStatusResponse.model_validate(response_data)


    def upload(self, body: Optional[UploadRequest] = None) -> UploadResponse:
        """
        Upload media
        Uploads a media file for use in posts or other content.
        body: Request body
        Returns:
            UploadResponse: Response data
        """
        url = self.client.base_url + "/2/media/upload"
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        headers = {}
        headers["Content-Type"] = "application/json"
        # Prepare request data
        json_data = None
        if body is not None:
            json_data = (
                body.model_dump(exclude_none=True)
                if hasattr(body, "model_dump")
                else body
            )
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        else:
            response = self.client.session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return UploadResponse.model_validate(response_data)


    def get_analytics(
        self,
        media_keys: List,
        end_time: str,
        start_time: str,
        granularity: str,
        media_analytics_fields: List = None,
    ) -> GetAnalyticsResponse:
        """
        Get Media analytics
        Retrieves analytics data for media.
        Args:
            media_keys: A comma separated list of Media Keys. Up to 100 are allowed in a single request.
            end_time: YYYY-MM-DDTHH:mm:ssZ. The UTC timestamp representing the end of the time range.
            start_time: YYYY-MM-DDTHH:mm:ssZ. The UTC timestamp representing the start of the time range.
            granularity: The granularity for the search counts results.
            media_analytics_fields: A comma separated list of MediaAnalytics fields to display.
            Returns:
            GetAnalyticsResponse: Response data
        """
        url = self.client.base_url + "/2/media/analytics"
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        if media_keys is not None:
            params["media_keys"] = ",".join(str(item) for item in media_keys)
        if end_time is not None:
            params["end_time"] = end_time
        if start_time is not None:
            params["start_time"] = start_time
        if granularity is not None:
            params["granularity"] = granularity
        if media_analytics_fields is not None:
            params["media_analytics.fields"] = ",".join(
                str(item) for item in media_analytics_fields
            )
        headers = {}
        # Prepare request data
        json_data = None
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.get(
                url,
                params=params,
                headers=headers,
            )
        else:
            response = self.client.session.get(
                url,
                params=params,
                headers=headers,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return GetAnalyticsResponse.model_validate(response_data)


    def create_subtitles(
        self, body: Optional[CreateSubtitlesRequest] = None
    ) -> CreateSubtitlesResponse:
        """
        Create Media subtitles
        Creates subtitles for a specific Media file.
        body: Request body
        Returns:
            CreateSubtitlesResponse: Response data
        """
        url = self.client.base_url + "/2/media/subtitles"
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        headers = {}
        headers["Content-Type"] = "application/json"
        # Prepare request data
        json_data = None
        if body is not None:
            json_data = (
                body.model_dump(exclude_none=True)
                if hasattr(body, "model_dump")
                else body
            )
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        else:
            response = self.client.session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return CreateSubtitlesResponse.model_validate(response_data)


    def delete_subtitles(
        self, body: Optional[DeleteSubtitlesRequest] = None
    ) -> DeleteSubtitlesResponse:
        """
        Delete Media subtitles
        Deletes subtitles for a specific Media file.
        body: Request body
        Returns:
            DeleteSubtitlesResponse: Response data
        """
        url = self.client.base_url + "/2/media/subtitles"
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        headers = {}
        headers["Content-Type"] = "application/json"
        # Prepare request data
        json_data = None
        if body is not None:
            json_data = (
                body.model_dump(exclude_none=True)
                if hasattr(body, "model_dump")
                else body
            )
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.delete(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        else:
            response = self.client.session.delete(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return DeleteSubtitlesResponse.model_validate(response_data)


    def create_metadata(
        self, body: Optional[CreateMetadataRequest] = None
    ) -> CreateMetadataResponse:
        """
        Create Media metadata
        Creates metadata for a Media file.
        body: Request body
        Returns:
            CreateMetadataResponse: Response data
        """
        url = self.client.base_url + "/2/media/metadata"
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        headers = {}
        headers["Content-Type"] = "application/json"
        # Prepare request data
        json_data = None
        if body is not None:
            json_data = (
                body.model_dump(exclude_none=True)
                if hasattr(body, "model_dump")
                else body
            )
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        else:
            response = self.client.session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return CreateMetadataResponse.model_validate(response_data)


    def get_by_keys(
        self, media_keys: List, media_fields: List = None
    ) -> GetByKeysResponse:
        """
        Get Media by media keys
        Retrieves details of Media files by their media keys.
        Args:
            media_keys: A comma separated list of Media Keys. Up to 100 are allowed in a single request.
            media_fields: A comma separated list of Media fields to display.
            Returns:
            GetByKeysResponse: Response data
        """
        url = self.client.base_url + "/2/media"
        if self.client.bearer_token:
            self.client.session.headers["Authorization"] = (
                f"Bearer {self.client.bearer_token}"
            )
        elif self.client.access_token:
            self.client.session.headers["Authorization"] = (
                f"Bearer {self.client.access_token}"
            )
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        if media_keys is not None:
            params["media_keys"] = ",".join(str(item) for item in media_keys)
        if media_fields is not None:
            params["media.fields"] = ",".join(str(item) for item in media_fields)
        headers = {}
        # Prepare request data
        json_data = None
        # Make the request
        response = self.client.session.get(
            url,
            params=params,
            headers=headers,
        )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return GetByKeysResponse.model_validate(response_data)


    def initialize_upload(
        self, body: Optional[InitializeUploadRequest] = None
    ) -> InitializeUploadResponse:
        """
        Initialize media upload
        Initializes a media upload.
        body: Request body
        Returns:
            InitializeUploadResponse: Response data
        """
        url = self.client.base_url + "/2/media/upload/initialize"
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        headers = {}
        headers["Content-Type"] = "application/json"
        # Prepare request data
        json_data = None
        if body is not None:
            json_data = (
                body.model_dump(exclude_none=True)
                if hasattr(body, "model_dump")
                else body
            )
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        else:
            response = self.client.session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return InitializeUploadResponse.model_validate(response_data)


    def finalize_upload(self, id: Any) -> FinalizeUploadResponse:
        """
        Finalize Media upload
        Finalizes a Media upload request.
        Args:
            id: The media id of the targeted media to finalize.
            Returns:
            FinalizeUploadResponse: Response data
        """
        url = self.client.base_url + "/2/media/upload/{id}/finalize"
        url = url.replace("{id}", str(id))
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        headers = {}
        # Prepare request data
        json_data = None
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.post(
                url,
                params=params,
                headers=headers,
            )
        else:
            response = self.client.session.post(
                url,
                params=params,
                headers=headers,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return FinalizeUploadResponse.model_validate(response_data)


    def get_by_key(self, media_key: Any, media_fields: List = None) -> GetByKeyResponse:
        """
        Get Media by media key
        Retrieves details of a specific Media file by its media key.
        Args:
            media_key: A single Media Key.
            media_fields: A comma separated list of Media fields to display.
            Returns:
            GetByKeyResponse: Response data
        """
        url = self.client.base_url + "/2/media/{media_key}"
        url = url.replace("{media_key}", str(media_key))
        if self.client.bearer_token:
            self.client.session.headers["Authorization"] = (
                f"Bearer {self.client.bearer_token}"
            )
        elif self.client.access_token:
            self.client.session.headers["Authorization"] = (
                f"Bearer {self.client.access_token}"
            )
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        if media_fields is not None:
            params["media.fields"] = ",".join(str(item) for item in media_fields)
        headers = {}
        # Prepare request data
        json_data = None
        # Make the request
        response = self.client.session.get(
            url,
            params=params,
            headers=headers,
        )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return GetByKeyResponse.model_validate(response_data)


    def append_upload(
        self, id: Any, body: Optional[AppendUploadRequest] = None
    ) -> AppendUploadResponse:
        """
        Append Media upload
        Appends data to a Media upload request.
        Args:
            id: The media identifier for the media to perform the append operation.
            body: Request body
        Returns:
            AppendUploadResponse: Response data
        """
        url = self.client.base_url + "/2/media/upload/{id}/append"
        url = url.replace("{id}", str(id))
        # Ensure we have a valid access token
        if self.client.oauth2_auth and self.client.token:
            # Check if token needs refresh
            if self.client.is_token_expired():
                self.client.refresh_token()
        params = {}
        headers = {}
        headers["Content-Type"] = "application/json"
        # Prepare request data
        json_data = None
        if body is not None:
            json_data = (
                body.model_dump(exclude_none=True)
                if hasattr(body, "model_dump")
                else body
            )
        # Make the request
        if self.client.oauth2_session:
            response = self.client.oauth2_session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        else:
            response = self.client.session.post(
                url,
                params=params,
                headers=headers,
                json=json_data,
            )
        # Check for errors
        response.raise_for_status()
        # Parse the response data
        response_data = response.json()
        # Convert to Pydantic model if applicable
        return AppendUploadResponse.model_validate(response_data)
